@@ -1,6 +1,6 @@
 # coding=utf-8
 # pynput
-# Copyright (C) 2015-2022 Moses Palmér
+# Copyright (C) 2015-2024 Moses Palmér
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -27,9 +27,11 @@ General utility functions and classes.
 import contextlib
 import functools
 import importlib
+import inspect
 import os
 import sys
 import threading
+import time
 
 import six
 
@@ -157,7 +159,7 @@ class AbstractListener(threading.Thread):
         self.daemon = True
 
         for name, callback in kwargs.items():
-            setattr(self, name, wrapper(callback or (lambda *a: None)))
+            setattr(self, name, wrapper(callback))
 
     @property
     def suppress(self):
@@ -265,15 +267,46 @@ class AbstractListener(threading.Thread):
         """
         raise NotImplementedError()
 
-    def join(self, *args):
-        super(AbstractListener, self).join(*args)
+    def _wrap(self, f, args):
+        """Wraps a callable to make it accept ``args`` number of arguments.
 
-        # Reraise any exceptions
+        :param f: The callable to wrap. If this is ``None`` a no-op wrapper is
+            returned.
+
+        :param int args: The number of arguments to accept.
+
+        :raises ValueError: if f requires more than ``args`` arguments
+        """
+        if f is None:
+            return lambda *a: None
+        else:
+            argspec = inspect.getfullargspec(f)
+            actual = len(inspect.signature(f).parameters)
+            defaults = len(argspec.defaults) if argspec.defaults else 0
+            if actual - defaults > args:
+                raise ValueError(f)
+            elif actual >= args or argspec.varargs is not None:
+                return f
+            else:
+                return lambda *a: f(*a[:actual])
+
+    def join(self, timeout=None, *args):
+        start = time.time()
+        super(AbstractListener, self).join(timeout, *args)
+        timeout = max(0.0, timeout - (time.time() - start)) \
+            if timeout is not None \
+            else None
+
+        # Reraise any exceptions; make sure not to block if a timeout was
+        # provided
         try:
-            exc_type, exc_value, exc_traceback = self._queue.get()
+            exc_type, exc_value, exc_traceback = self._queue.get(
+                timeout=timeout)
+            six.reraise(exc_type, exc_value, exc_traceback)
+        except queue.Empty:
+            pass
         except TypeError:
             return
-        six.reraise(exc_type, exc_value, exc_traceback)
 
 
 class Events(object):
