@@ -1,6 +1,6 @@
 # coding=utf-8
 # pynput
-# Copyright (C) 2015-2022 Moses Palmér
+# Copyright (C) 2015-2024 Moses Palmér
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -79,17 +79,23 @@ class Events(Events):
     class Press(Events.Event):
         """A key press event.
         """
-        def __init__(self, key):
+        def __init__(self, key, injected):
             #: The key.
             self.key = key
 
+            #: Whether this event is synthetic.
+            self.injected = injected
+
     class Release(Events.Event):
         """A key release event.
         """
-        def __init__(self, key):
+        def __init__(self, key, injected):
             #: The key.
             self.key = key
 
+            #: Whether this event is synthetic.
+            self.injected = injected
+
     def __init__(self):
         super(Events, self).__init__(
             on_press=self.Press,
@@ -144,7 +150,13 @@ class HotKey(object):
             elif len(s) > 2 and (s[0], s[-1]) == ('<', '>'):
                 p = s[1:-1]
                 try:
-                    return Key[p.lower()]
+                    # We want to represent modifiers as Key instances, and all
+                    # other keys as KeyCodes
+                    key = Key[p.lower()]
+                    if key in _NORMAL_MODIFIERS.values():
+                        return key
+                    else:
+                        return KeyCode.from_vk(key.value.vk)
                 except KeyError:
                     try:
                         return KeyCode.from_vk(int(p))
@@ -212,22 +224,26 @@ class GlobalHotKeys(Listener):
             *args,
             **kwargs)
 
-    def _on_press(self, key):
+    def _on_press(self, key, injected):
         """The press callback.
 
         This is automatically registered upon creation.
 
         :param key: The key provided by the base class.
+        :param injected: Whether the event was injected.
         """
-        for hotkey in self._hotkeys:
-            hotkey.press(self.canonical(key))
+        if not injected:
+            for hotkey in self._hotkeys:
+                hotkey.press(self.canonical(key))
 
-    def _on_release(self, key):
+    def _on_release(self, key, injected):
         """The release callback.
 
         This is automatically registered upon creation.
 
         :param key: The key provided by the base class.
+        :param injected: Whether the event was injected.
         """
-        for hotkey in self._hotkeys:
-            hotkey.release(self.canonical(key))
+        if not injected:
+            for hotkey in self._hotkeys:
+                hotkey.release(self.canonical(key))
