@@ -1,6 +1,6 @@
 # coding=utf-8
 # pynput
-# Copyright (C) 2015-2022 Moses Palmér
+# Copyright (C) 2015-2024 Moses Palmér
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -205,7 +205,7 @@ class Controller(NotifierMixin, _base.Controller):
         # pylint: enable=C0103
 
     def __del__(self):
-        if self._display:
+        if hasattr(self, '_display'):
             self._display.close()
 
     @property
@@ -223,9 +223,8 @@ class Controller(NotifierMixin, _base.Controller):
     def _handle(self, key, is_press):
         """Resolves a key identifier and sends a keyboard event.
 
-        :param event: The *X* keyboard event.
-
-        :param int keysym: The keysym to handle.
+        :param int key: The key to handle.
+        :param bool is_press: Whether this is a press.
         """
         event = Xlib.display.event.KeyPress if is_press \
             else Xlib.display.event.KeyRelease
@@ -414,11 +413,26 @@ class Controller(NotifierMixin, _base.Controller):
         #: Registers a keycode for a specific key and modifier state
         def register(dm, keycode, index):
             i = kc2i(keycode)
-            mapping[i][index] = keysym
-            dm.change_keyboard_mapping(
-                keycode,
-                mapping[i:i + 1])
-            self._borrows[keysym] = (keycode, index, 0)
+
+            # Check for use of empty mapping with a character that has upper
+            # and lower forms
+            lower = key.char.lower()
+            upper = key.char.upper()
+            if lower != upper and len(lower) == 1 and len(upper) == 1 and all(
+                    m == Xlib.XK.NoSymbol
+                    for m in mapping[i]):
+                lower = self._key_to_keysym(KeyCode.from_char(lower))
+                upper = self._key_to_keysym(KeyCode.from_char(upper))
+                if lower:
+                    mapping[i][0] = lower
+                    self._borrows[lower] = (keycode, 0, 0)
+                if upper:
+                    mapping[i][1] = upper
+                    self._borrows[upper] = (keycode, 1, 0)
+            else:
+                mapping[i][index] = keysym
+                self._borrows[keysym] = (keycode, index, 0)
+            dm.change_keyboard_mapping(keycode, mapping[i:i + 1])
 
         try:
             with display_manager(self._display) as dm, self._borrow_lock as _:
@@ -545,14 +559,14 @@ class Listener(ListenerMixin, _base.Listener):
             super(Listener, self)._run()
 
     def _initialize(self, display):
-        # Get the keyboard mapping to be able to translate events details to
+        # Get the keyboard mapping to be able to translate event details to
         # key codes
         min_keycode = display.display.info.min_keycode
         keycode_count = display.display.info.max_keycode - min_keycode + 1
         self._keyboard_mapping = display.get_keyboard_mapping(
             min_keycode, keycode_count)
 
-    def _handle(self, display, event):
+    def _handle_message(self, display, event, injected):
         # Convert the event to a KeyCode; this may fail, and in that case we
         # pass None
         try:
@@ -561,10 +575,10 @@ class Listener(ListenerMixin, _base.Listener):
             key = None
 
         if event.type == Xlib.X.KeyPress:
-            self.on_press(key)
+            self.on_press(key, injected)
 
         elif event.type == Xlib.X.KeyRelease:
-            self.on_release(key)
+            self.on_release(key, injected)
 
     def _suppress_start(self, display):
         display.screen().root.grab_keyboard(
@@ -582,7 +596,7 @@ class Listener(ListenerMixin, _base.Listener):
         :param bool is_press: Whether this is a press event.
         """
         (self.on_press if is_press else self.on_release)(
-            self._SPECIAL_KEYS.get(key.vk, key))
+            self._SPECIAL_KEYS.get(key.vk, key), True)
 
     def _keycode_to_keysym(self, display, keycode, index):
         """Converts a keycode and shift state index to a keysym.
